@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018 Alexander Borisov
+ * Copyright (C) 2018-2020 Alexander Borisov
  *
  * Author: Alexander Borisov <borisov@lexbor.com>
  */
@@ -533,11 +533,11 @@ lxb_dom_attr_t *
 lxb_dom_element_attr_is_exist(lxb_dom_element_t *element,
                               const lxb_char_t *qualified_name, size_t length)
 {
-    lxb_dom_attr_data_t *data;
+    const lxb_dom_attr_data_t *data;
     lxb_dom_attr_t *attr = element->first_attr;
 
-    data = lexbor_hash_search(element->node.owner_document->attrs,
-                          lexbor_hash_search_lower, qualified_name, length);
+    data = lxb_dom_attr_data_by_local_name(element->node.owner_document->attrs,
+                                           qualified_name, length);
     if (data == NULL) {
         return NULL;
     }
@@ -724,7 +724,7 @@ static lexbor_action_t
 lxb_dom_elements_by_tag_name_cb_all(lxb_dom_node_t *node, void *ctx)
 {
     if (node->type != LXB_DOM_NODE_TYPE_ELEMENT) {
-        return LEXBOR_ACTION_NEXT;
+        return LEXBOR_ACTION_OK;
     }
 
     lxb_dom_element_cb_ctx_t *cb_ctx = ctx;
@@ -734,14 +734,14 @@ lxb_dom_elements_by_tag_name_cb_all(lxb_dom_node_t *node, void *ctx)
         return LEXBOR_ACTION_STOP;
     }
 
-    return LEXBOR_ACTION_NEXT;
+    return LEXBOR_ACTION_OK;
 }
 
 static lexbor_action_t
 lxb_dom_elements_by_tag_name_cb(lxb_dom_node_t *node, void *ctx)
 {
     if (node->type != LXB_DOM_NODE_TYPE_ELEMENT) {
-        return LEXBOR_ACTION_NEXT;
+        return LEXBOR_ACTION_OK;
     }
 
     lxb_dom_element_cb_ctx_t *cb_ctx = ctx;
@@ -755,7 +755,7 @@ lxb_dom_elements_by_tag_name_cb(lxb_dom_node_t *node, void *ctx)
         }
     }
 
-    return LEXBOR_ACTION_NEXT;
+    return LEXBOR_ACTION_OK;
 }
 
 lxb_status_t
@@ -783,7 +783,7 @@ static lexbor_action_t
 lxb_dom_elements_by_class_name_cb(lxb_dom_node_t *node, void *ctx)
 {
     if (node->type != LXB_DOM_NODE_TYPE_ELEMENT) {
-        return LEXBOR_ACTION_NEXT;
+        return LEXBOR_ACTION_OK;
     }
 
     lxb_dom_element_cb_ctx_t *cb_ctx = ctx;
@@ -792,7 +792,7 @@ lxb_dom_elements_by_class_name_cb(lxb_dom_node_t *node, void *ctx)
     if (el->attr_class == NULL
         || el->attr_class->value->length < cb_ctx->value_length)
     {
-        return LEXBOR_ACTION_NEXT;
+        return LEXBOR_ACTION_OK;
     }
 
     const lxb_char_t *data = el->attr_class->value->data;
@@ -804,11 +804,10 @@ lxb_dom_elements_by_class_name_cb(lxb_dom_node_t *node, void *ctx)
 
     lxb_dom_document_t *doc = el->node.owner_document;
 
-    for (; (end - data) >= cb_ctx->value_length; data++)
-    {
+    for (; data < end; data++) {
         if (lexbor_utils_whitespace(*data, ==, ||)) {
-            if (pos != data && (data - pos) == cb_ctx->value_length)
-            {
+
+            if (pos != data && (data - pos) == cb_ctx->value_length) {
                 if (doc->compat_mode == LXB_DOM_DOCUMENT_CMODE_QUIRKS) {
                     is_it = lexbor_str_data_ncasecmp(pos, cb_ctx->value,
                                                      cb_ctx->value_length);
@@ -825,10 +824,14 @@ lxb_dom_elements_by_class_name_cb(lxb_dom_node_t *node, void *ctx)
                         return LEXBOR_ACTION_STOP;
                     }
 
-                    return LEXBOR_ACTION_NEXT;
+                    return LEXBOR_ACTION_OK;
                 }
             }
 
+            if ((end - data) < cb_ctx->value_length) {
+                return LEXBOR_ACTION_OK;
+            }
+
             pos = data + 1;
         }
     }
@@ -851,7 +854,7 @@ lxb_dom_elements_by_class_name_cb(lxb_dom_node_t *node, void *ctx)
         }
     }
 
-    return LEXBOR_ACTION_NEXT;
+    return LEXBOR_ACTION_OK;
 }
 
 lxb_status_t
@@ -1006,7 +1009,7 @@ static lexbor_action_t
 lxb_dom_elements_by_attr_cb(lxb_dom_node_t *node, void *ctx)
 {
     if (node->type != LXB_DOM_NODE_TYPE_ELEMENT) {
-        return LEXBOR_ACTION_NEXT;
+        return LEXBOR_ACTION_OK;
     }
 
     lxb_dom_attr_t *attr;
@@ -1015,7 +1018,7 @@ lxb_dom_elements_by_attr_cb(lxb_dom_node_t *node, void *ctx)
 
     attr = lxb_dom_element_attr_by_id(el, cb_ctx->name_id);
     if (attr == NULL) {
-        return LEXBOR_ACTION_NEXT;
+        return LEXBOR_ACTION_OK;
     }
 
     if ((cb_ctx->value_length == 0 && attr->value->length == 0)
@@ -1028,7 +1031,7 @@ lxb_dom_elements_by_attr_cb(lxb_dom_node_t *node, void *ctx)
         }
     }
 
-    return LEXBOR_ACTION_NEXT;
+    return LEXBOR_ACTION_OK;
 }
 
 static bool
