@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018-2019 Alexander Borisov
+ * Copyright (C) 2018-2020 Alexander Borisov
  *
  * Author: Alexander Borisov <borisov@lexbor.com>
  */
@@ -14,8 +14,8 @@ extern "C" {
 #include "lexbor/core/base.h"
 
 
-#define LXB_HTML_VERSION_MAJOR 1
-#define LXB_HTML_VERSION_MINOR 4
+#define LXB_HTML_VERSION_MAJOR 2
+#define LXB_HTML_VERSION_MINOR 1
 #define LXB_HTML_VERSION_PATCH 0
 
 #define LXB_HTML_VERSION_STRING LEXBOR_STRINGIZE(LXB_HTML_VERSION_MAJOR) "."   \
@@ -23,6 +23,8 @@ extern "C" {
                                 LEXBOR_STRINGIZE(LXB_HTML_VERSION_PATCH)
 
 
+typedef struct lxb_html_tokenizer lxb_html_tokenizer_t;
+typedef unsigned int lxb_html_tokenizer_opt_t;
 typedef struct lxb_html_tree lxb_html_tree_t;
 
 /*
